# -*- coding: utf-8 -

"""Test the created constraints against approved constraints.

This file is part of project oemof (github.com/oemof/oemof). It's copyrighted
by the contributors recorded in the version control history of the file,
available from its original location oemof/tests/tool_tests.py

SPDX-License-Identifier: MIT
"""

import logging
import os
import warnings

import pytest

from oemof.tools import debugging
from oemof.tools import economics
from oemof.tools.logger import define_logging
from oemof.tools.logger import extend_basic_path
from oemof.tools.logger import get_basic_path


def test_helpers():
    get_basic_path()
    assert os.path.isdir(os.path.join(os.path.expanduser("~"), ".oemof"))
    new_dir = extend_basic_path("test_xf67456_dir")
    assert os.path.isdir(new_dir)
    get_basic_path()  # get basic path should not overwrite path
    assert os.path.isdir(new_dir)
    os.rmdir(new_dir)
    assert not os.path.isdir(new_dir)


def test_define_logging_format():
    file_format = "%(module)s - %(message)s"
    screen_format = "%(levelname)s-%(message)s"
    my_logpath = extend_basic_path("test_xf4hz4345456_dir")
    my_logfile = define_logging(
        logpath=my_logpath,
        log_path=False,
        screen_format=screen_format,
        file_format=file_format,
        file_level=logging.INFO,
    )
    logging.info("basdfuio")
    f = open(my_logfile, "r")
    log_content = f.read()
    assert "test_tools - basdfuio" in log_content
    assert "DEBUG" not in log_content
    os.remove(my_logfile)
    os.rmdir(my_logpath)


def test_logg_file_in_new_path():
    my_logpath = extend_basic_path("test_xf4hz4u67456_dir")
    my_logfile = define_logging(logpath=my_logpath, file_level=logging.DEBUG)
    assert os.path.join(my_logpath, "oemof.log") == my_logfile
    assert os.path.isfile(my_logfile)
    logging.debug("Tester345")
    f = open(my_logfile, "r")
    log_content = f.read()
    assert "Tester345" in log_content
    assert "DEBUG" in log_content
    os.remove(my_logfile)
    os.rmdir(my_logpath)


def test_logger():
    filepath = define_logging()
    assert isinstance(filepath, str)
    assert filepath[-9:] == "oemof.log"
    assert os.path.isfile(filepath)


def test_annuity():
    """Test annuity function of economics tool."""
    assert round(economics.annuity(1000, 10, 0.1)) == 163
    assert round(economics.annuity(capex=1000, wacc=0.1, n=10, u=5)) == 264
    assert (
        round(economics.annuity(1000, 10, 0.1, u=5, cost_decrease=0.1)) == 222
    )


def test_annuity_exceptions():
    """Test out-of-bounds-error of the annuity tool."""
    pytest.raises(ValueError, economics.annuity, 1000, 10, 2)
    pytest.raises(ValueError, economics.annuity, 1000, 0.5, 1)
    pytest.raises(ValueError, economics.annuity, 1000, 10, 0.1, u=0.3)
    pytest.raises(
        ValueError, economics.annuity, 1000, 10, 0.1, cost_decrease=-1
    )


def test_suspicious_usage_warning():
    msg = "My message."
    with warnings.catch_warnings(record=True) as w:
        warnings.warn(msg, debugging.SuspiciousUsageWarning)
        assert len(w) == 1
        assert msg, str(w[-1].message)



# -*- coding: utf-8 -*-

"""Module contains tools facilitating debugging

This file is part of project oemof (github.com/oemof/oemof). It's copyrighted
by the contributors recorded in the version control history of the file,
available from its original location oemof/oemof/tools/economics.py

SPDX-License-Identifier: MIT
"""


class SuspiciousUsageWarning(UserWarning):
    """
    Warn the user about potentially dangerous usage.

    Some ways of using `oemof` are not necessarily wrong but could lead to
    hard to find bugs if you done accidentally instead of intentionally. We
    use these warnings, and you can do too ;), in your code to warn users about
    these cases. If you know what you are doing and these warnings point you to
    things you are doing intentionally, you can easily switch them off.

    Note
    ----
    TODO: Fix ref!
    See :ref:`oemof_tools_debugging_suspicioususagewarningsolph_label` for more
    information.

    Examples
    --------
    >>> import warnings
    >>> warnings.filterwarnings("ignore", category=SuspiciousUsageWarning)
    """

    pass



# -*- coding: utf-8

"""Helpers to log your modeling process with oemof.

This file is part of project oemof (github.com/oemof/oemof). It's copyrighted
by the contributors recorded in the version control history of the file,
available from its original location oemof/oemof/tools/logger.py

SPDX-License-Identifier: MIT
"""

import os
import sys
from logging import INFO
from logging import WARNING
from logging import Formatter
from logging import StreamHandler
from logging import debug
from logging import getLogger
from logging import handlers
from logging import info


def define_logging(
    logpath=None,
    logfile="oemof.log",
    file_format=None,
    screen_format=None,
    file_datefmt=None,
    screen_datefmt=None,
    screen_level=INFO,
    file_level=WARNING,
    log_path=True,
    timed_rotating=None,
):

    r"""Initialise customisable logger.

    Parameters
    ----------
    logfile : str
        Name of the log file, default: oemof.log
    logpath : str
        The path for log files. By default a ".oemof' folder is created in your
        home directory with subfolder called 'log_files'.
    file_format : str
        Format of the file output.
        Default: "%(asctime)s - %(levelname)s - %(module)s - %(message)s"
    screen_format : str
        Format of the screen output.
        Default: "%(asctime)s-%(levelname)s-%(message)s"
    file_datefmt : str
        Format of the datetime in the file output. Default: None
    screen_datefmt : str
        Format of the datetime in the screen output. Default: "%H:%M:%S"
    screen_level : int
        Level of logging to stdout. Default: 20 (logging.INFO)
    file_level : int
        Level of logging to file. Default: 30 (logging.WARNING)
    log_path : boolean
        If True the used file path is logged while initialising the logger.
    timed_rotating : dict
        Option to pass parameters to the TimedRotatingFileHandler.


    Returns
    -------
    str : Place where the log file is stored.

    Notes
    -----
    By default the INFO level is printed on the screen and the DEBUG level
    in a file, but you can easily configure the logger.
    Every module that wants to create logging messages has to import the
    logging module. The oemof logger module has to be imported once to
    initialise it.

    Examples
    --------
    To define the default logger you have to import the python logging
    library and this function. The first logging message should be the
    path where the log file is saved to.

    >>> import logging
    >>> from oemof.tools import logger
    >>> mypath = logger.define_logging(
    ...     log_path=True, timed_rotating={'backupCount': 4},
    ...     screen_level=logging.ERROR, screen_datefmt = "no_date")
    >>> mypath[-9:]
    'oemof.log'
    >>> logging.debug("Hallo")
    """

    if logpath is None:
        logpath = extend_basic_path("log_files")

    # Fetch minimal logging level
    if screen_level > file_level:
        min_level = file_level
    else:
        min_level = screen_level

    file = os.path.join(logpath, logfile)

    log = getLogger("")

    # Remove existing handlers to avoid interference.
    log.handlers = []
    log.setLevel(min_level)

    if file_format is None:
        file_format = "%(asctime)s - %(levelname)s - %(module)s - %(message)s"
    file_formatter = Formatter(file_format, file_datefmt)

    if screen_format is None:
        screen_format = "%(asctime)s-%(levelname)s-%(message)s"
    if screen_datefmt is None:
        screen_datefmt = "%H:%M:%S"
    screen_formatter = Formatter(screen_format, screen_datefmt)

    tmp_formatter = Formatter("%(message)s")

    ch = StreamHandler(sys.stdout)
    ch.setFormatter(screen_formatter)
    ch.setLevel(screen_level)
    log.addHandler(ch)

    timed_rotating_p = {"when": "midnight", "backupCount": 10}

    if timed_rotating is not None:
        timed_rotating_p.update(timed_rotating)

    fh = handlers.TimedRotatingFileHandler(file, **timed_rotating_p)
    fh.setFormatter(tmp_formatter)
    fh.setLevel(file_level)
    log.addHandler(fh)

    debug("******************************************************")
    fh.setFormatter(file_formatter)
    if log_path:
        info("Path for logging: {0}".format(file))
    return file


def extend_basic_path(subfolder):
    """Returns a path based on the basic oemof path and creates it if
    necessary. The subfolder is the name of the path extension.
    """
    extended_path = os.path.join(get_basic_path(), subfolder)
    if not os.path.isdir(extended_path):
        os.mkdir(extended_path)
    return extended_path


def get_basic_path():
    """Returns the basic oemof path and creates it if necessary.
    The basic path is the '.oemof' folder in the $HOME directory.
    """
    basicpath = os.path.join(os.path.expanduser("~"), ".oemof")
    if not os.path.isdir(basicpath):
        os.mkdir(basicpath)
    return basicpath



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import io
import re
from glob import glob
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    with io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8"),
    ) as fh:
        return fh.read()


long_description = "%s\n%s" % (
    re.compile("^.. start-badges.*^.. end-badges", re.M | re.S).sub(
        "", read("README.rst")
    ),
    re.sub(":[a-z]+:`~?(.*?)`", r"``\1``", read("CHANGELOG.rst")),
)

setup(
    name="oemof.tools",
    version="0.4.2",
    license="MIT",
    description="Tiny tools of the oemof project.",
    long_description_content_type="text/x-rst",
    long_description=long_description,
    author="oemof-developer-group",
    author_email="contact@oemof.org",
    url="https://github.com/oemof/oemof-tools",
    packages=["oemof"] + ["oemof." + p for p in find_packages("src/oemof")],
    package_dir={"": "src"},
    py_modules=[splitext(basename(path))[0] for path in glob("src/*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list:
        # http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: Implementation :: CPython",
        "Topic :: Utilities",
    ],
    project_urls={
        "Documentation": "https://oemof-tools.readthedocs.io/",
        "Changelog": (
            "https://oemof-tools.readthedocs.io/en/latest/changelog.html"
        ),
        "Issue Tracker": "https://github.com/oemof/oemof-tools/issues",
    },
    keywords=[
        # eg: 'keyword1', 'keyword2', 'keyword3',
    ],
    python_requires=">=3.8",
    install_requires=[
        # eg: 'aspectlib==1.1.1', 'six>=1.7',
    ],
    extras_require={
        "dev": [
            "pytest",
            "sphinx",
            "sphinx_rtd_theme",
        ]
    },
)



# -*- coding: utf-8 -*-

"""Module to collect useful functions for economic calculation.

This file is part of project oemof (github.com/oemof/oemof). It's copyrighted
by the contributors recorded in the version control history of the file,
available from its original location oemof/oemof/tools/economics.py

SPDX-License-Identifier: MIT
"""


def annuity(capex, n, wacc, u=None, cost_decrease=0):
    r"""Calculates the annuity of an initial investment 'capex', considering
    the cost of capital 'wacc' during a project horizon 'n'

    In case of a single initial investment, the employed formula reads:

    .. math::
        \text{annuity} = \text{capex} \cdot
            \frac{(\text{wacc} \cdot (1+\text{wacc})^n)}
            {((1 + \text{wacc})^n - 1)}

    In case of repeated investments (due to replacements) at fixed intervals
    'u', the formula yields:

    .. math::
        \text{annuity} = \text{capex} \cdot
                  \frac{(\text{wacc} \cdot (1+\text{wacc})^n)}
                  {((1 + \text{wacc})^n - 1)} \cdot \left(
                  \frac{1 - \left( \frac{(1-\text{cost\_decrease})}
                  {(1+\text{wacc})} \right)^n}
                  {1 - \left(\frac{(1-\text{cost\_decrease})}{(1+\text{wacc})}
                  \right)^u} \right)

    Parameters
    ----------
    capex : float
        Capital expenditure for first investment. Net Present Value (NPV) or
        Net Present Cost (NPC) of investment
    n : int
        Horizon of the analysis, or number of years the annuity wants to be
        obtained for (n>=1)
    wacc : float
        Weighted average cost of capital (0<wacc<1)
    u : int
        Lifetime of the investigated investment. Might be smaller than the
        analysis horizon, 'n', meaning it will have to be replaced.
        Takes value 'n' if not specified otherwise (u>=1)
    cost_decrease : float
        Annual rate of cost decrease (due to, e.g., price experience curve).
        This only influences the result for investments corresponding to
        replacements, whenever u<n.
        Takes value 0, if not specified otherwise (0<cost_decrease<1)
    Returns
    -------
    float
        annuity
    """
    if u is None:
        u = n

    if (
        (n < 1)
        or (wacc < 0 or wacc > 1)
        or (u < 1)
        or (cost_decrease < 0 or cost_decrease > 1)
    ):
        raise ValueError("Input arguments for 'annuity' out of bounds!")

    return (
        capex
        * (wacc * (1 + wacc) ** n)
        / ((1 + wacc) ** n - 1)
        * (
            (1 - ((1 - cost_decrease) / (1 + wacc)) ** n)
            / (1 - ((1 - cost_decrease) / (1 + wacc)) ** u)
        )
    )
